package qwazlab.geom
{
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import qwazlab.geom.Edge;
	
	/**
	 * ...
	 * @author Qwaz
	 */
	public class Face extends Sprite
	{
		private var _vertices:Vector.<Point>;
		private var _edges:Vector.<Edge> = new Vector.<Edge>();
		
		private var _canvas:Shape = new Shape();
		
		public function Face () {
			this.addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init (e:Event):void {
			this.removeEventListener(Event.ADDED_TO_STAGE, init);
			this.addChild(_canvas);
		}
		
		public function attachToOtherFace (target:Edge, to:Edge):void {
			if (target.parent != this) {
				throw new ArgumentError("target must be the child of caller");
			}
			
			var fiStart:Point = this.localToGlobal(target.start);
			var fiEnd:Point = this.localToGlobal(target.end);
			var fiDistance:Number = Point.distance(fiStart, fiEnd);
			
			var seStart:Point = to.parent.localToGlobal(to.start);
			var seEnd:Point = to.parent.localToGlobal(to.end);
			var seDistance:Number = Point.distance(seStart, seEnd);
			
			this.scaleX *= seDistance / fiDistance;
			this.scaleY *= seDistance / fiDistance;
			
			var targetAngle:Number = Math.atan2(seEnd.y - seStart.y, seEnd.x - seStart.x) * 180 / Math.PI;
			var nowAngle:Number = Math.atan2(fiStart.y - fiEnd.y, fiStart.x - fiEnd.x) * 180 / Math.PI;
			
			if (targetAngle > nowAngle) {
				targetAngle += 360;
			}
			this.rotation += targetAngle - nowAngle;
			
			fiStart = this.localToGlobal(target.start);
			fiEnd = this.localToGlobal(target.end);
			
			this.x += seEnd.x - fiStart.x;
			this.y += seEnd.y - fiStart.y;
			
			target._linked = to;
			to._linked = target;
		}
		
		public static function makeRegularPolygon(n:uint, length:Number):Face {
			var vertices:Vector.<Point> = new Vector.<Point>();
			var i:int;
			for (i = 0; i < n; i++) {
				vertices.push(Point.polar(Math.sqrt((length * length) / (2 - 2 * Math.cos(2 * Math.PI / n))), 2 * Math.PI / n * i - Math.PI / 2));
			}
			
			return makeByVertices(vertices);
		}
		
		public static function makeByVertices(vertices:Vector.<Point>):Face {
			if (vertices.length < 3) {
				throw new ArgumentError("Require more than 3 vertices");
			}
			
			var face:Face = new Face();
			var point:Point;
			var i:int;
			with (face._canvas.graphics) {
				if (_nowFilling) {
					beginFill(_fillColor, _fillAlpha);
				}
				if (_lineInitialized) {
					lineStyle(_thickness, _lineColor, _lineAlpha, _pixelHinting, _scaleMode, _caps, _joints, _miterLimit);
				}
				moveTo(vertices[0].x, vertices[0].y);
			}
			for (i = 1; i < vertices.length; i++) {
				face._canvas.graphics.lineTo(vertices[i].x, vertices[i].y);
				face._edges.push(new Edge(vertices[i - 1], vertices[i], face));
			}
			face._edges.push(new Edge(vertices[i - 1], vertices[0], face));
			face._canvas.graphics.lineTo(vertices[0].x, vertices[0].y);
			face._vertices = vertices;
			return face;
		}
		
		private static var _fillColor:uint;
		private static var _fillAlpha:Number;
		private static var _nowFilling:Boolean = false;
		
		public static function beginFill(color:uint, alpha:Number = 1):void {
			_fillColor = color;
			_fillAlpha = alpha;
			_nowFilling = true;
		}
		
		public static function endFill():void {
			_nowFilling = false;
		}
		
		private static var _thickness:Number;
		private static var _lineColor:uint;
		private static var _lineAlpha:Number;
		private static var _pixelHinting:Boolean;
		private static var _scaleMode:String;
		private static var _caps:String;
		private static var _joints:String;
		private static var _miterLimit:Number;
		private static var _lineInitialized:Boolean = false;
		
		public static function lineStyle(
		thickness:Number = 0,
		color:uint = 0,
		alpha:Number = 1,
		pixelHinting:Boolean = false,
		scaleMode:String = "normal",
		caps:String = null,
		joints:String = null,
		miterLimit:Number = 3):void {
			_thickness = thickness;
			_lineColor = color;
			_lineAlpha = alpha;
			_pixelHinting = pixelHinting;
			_scaleMode = scaleMode;
			_caps = caps;
			_joints = joints;
			_miterLimit = miterLimit;
			_lineInitialized = true;
		}
		
		public function get vertices():Vector.<Point> {
			return _vertices;
		}
		
		public function get edges():Vector.<Edge> {
			return _edges;
		}
	}
	
}